Index: include/asterisk/res_odbc.h
===================================================================
--- include/asterisk/res_odbc.h	(revision 82391)
+++ include/asterisk/res_odbc.h	(working copy)
@@ -93,6 +93,16 @@
  */
 int ast_odbc_sanity_check(struct odbc_obj *obj);
 
+/*! \brief Executes an non prepared statement and returns the resulting
+ * statement handle.
+ * \param obj The ODBC object
+ * \param exec_cb A function callback, which, when called, should return a statement handle with result columns bound.
+ * \param data A parameter to be passed to the exec_cb parameter function, indicating which statement handle is to be prepared.
+ * \retval a statement handle
+ * \retval NULL on error
+ */
+SQLHSTMT ast_odbc_direct_execute(struct odbc_obj *obj, SQLHSTMT (*exec_cb)(struct odbc_obj *obj, void *data), void *data);
+
 /*! 
  * \brief Prepares, executes, and returns the resulting statement handle.
  * \param obj The ODBC object
Index: res/res_odbc.c
===================================================================
--- res/res_odbc.c	(revision 82391)
+++ res/res_odbc.c	(working copy)
@@ -77,6 +77,28 @@
 static int odbc_register_class(struct odbc_class *class, int connect);
 
 
+SQLHSTMT ast_odbc_direct_execute(struct odbc_obj *obj, SQLHSTMT (*exec_cb)(struct odbc_obj *obj, void *data), void *data)
+{
+	int attempt;
+	SQLHSTMT stmt;
+
+	for (attempt = 0; attempt < 2; attempt++) {
+		stmt = exec_cb(obj, data);
+
+		if (stmt) {
+			break;
+		} else {
+			obj->up = 0;
+			ast_log(LOG_WARNING, "SQL Exec Direct failed.  Attempting a reconnect...\n");
+
+			odbc_obj_disconnect(obj);
+			odbc_obj_connect(obj);
+		}
+	}
+
+	return stmt;
+}
+
 SQLHSTMT ast_odbc_prepare_and_execute(struct odbc_obj *obj, SQLHSTMT (*prepare_cb)(struct odbc_obj *obj, void *data), void *data)
 {
 	int res = 0, i, attempt;
